import sys
import time
import threading
import tkinter as tk
from tkinter import StringVar

# Add the Yoctopuce library path
sys.path.append(r"C:\data\tmp\YoctoLib.python.62875\Sources")

from yocto_api import *
from yocto_temperature import *
from yocto_relay import *


def initialize_modules():
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        raise Exception(f"Initialization error: {errmsg.value}")


def find_temperature_sensor():
    return YTemperature.FindTemperature("METEOMK2-20713E.temperature")


def find_relay():
    return YRelay.FindRelay("MXPWRRLY-45C5B.relay1")


class TemperatureRelayApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Temperature Relay Controller")

        # Initialize variables
        self.temperature = StringVar(value="N/A")
        self.sensor_status = StringVar(value="Disconnected")
        self.relay_status = StringVar(value="Disconnected")
        self.relay_state = StringVar(value="Unknown")

        # Create UI elements
        tk.Label(root, text="Room Temperature:").grid(row=0, column=0, sticky="w")
        tk.Label(root, textvariable=self.temperature, font=("Arial", 16)).grid(row=0, column=1, sticky="w")

        tk.Label(root, text="Sensor Status:").grid(row=1, column=0, sticky="w")
        tk.Label(root, textvariable=self.sensor_status).grid(row=1, column=1, sticky="w")

        tk.Label(root, text="Relay Status:").grid(row=2, column=0, sticky="w")
        tk.Label(root, textvariable=self.relay_status).grid(row=2, column=1, sticky="w")

        tk.Label(root, text="Relay State:").grid(row=3, column=0, sticky="w")
        tk.Label(root, textvariable=self.relay_state).grid(row=3, column=1, sticky="w")

        # Initialize Yocto modules
        initialize_modules()
        self.temperature_sensor = find_temperature_sensor()
        self.relay = find_relay()

        # Start background thread
        self.running = True
        self.update_thread = threading.Thread(target=self.update_loop, daemon=True)
        self.update_thread.start()

    def update_loop(self):
        while self.running:
            try:
                # Refresh Yocto device list
                YAPI.UpdateDeviceList()

                # Update temperature sensor status and value
                if self.temperature_sensor and self.temperature_sensor.isOnline():
                    temp = self.temperature_sensor.get_currentValue()
                    self.temperature.set(f"{temp:.2f} °C")
                    self.sensor_status.set("Connected")

                    # Update relay status and state
                    if self.relay and self.relay.isOnline():
                        self.relay_status.set("Connected")

                        # Update relay state based on temperature
                        if temp > 25:
                            self.relay.set_state(YRelay.STATE_A)  # Open relay
                            self.relay_state.set("Open (temperature > 25°C)")
                        else:
                            self.relay.set_state(YRelay.STATE_B)  # Close relay
                            self.relay_state.set("Closed (temperature <= 25°C)")
                    else:
                        self.relay_status.set("Disconnected")
                        self.relay_state.set("Unknown")
                        self.relay = find_relay()
                else:
                    self.sensor_status.set("Disconnected")
                    self.temperature.set("N/A")
                    self.temperature_sensor = find_temperature_sensor()

            except YAPI_Exception as e:
                print(f"Yoctopuce API Error: {e}")

            time.sleep(1)

    def stop(self):
        self.running = False
        YAPI.FreeAPI()


if __name__ == "__main__":
    root = tk.Tk()
    app = TemperatureRelayApp(root)
    try:
        root.protocol("WM_DELETE_WINDOW", app.stop)  # Graceful shutdown
        root.mainloop()
    except KeyboardInterrupt:
        app.stop()
