import time
import threading
import tkinter as tk
from yocto_api import *
from yocto_temperature import *
from yocto_relay import *


class YoctoApp:
    def __init__(self, root):
        self.root = root
        self.root.title("Application Yoctopuce")
        self.root.geometry("400x200")

        self.temperature_sensor = None
        self.relay = None
        self.running = True

        # Initialisation des modules Yoctopuce
        self.init_modules()

        # Création des widgets graphiques
        self.temp_label = tk.Label(self.root, text="Température : -- °C", font=("Helvetica", 16))
        self.temp_label.pack(pady=10)

        self.relay_label = tk.Label(self.root, text="État du relais : --", font=("Helvetica", 16))
        self.relay_label.pack(pady=10)

        self.quit_button = tk.Button(self.root, text="Quitter", command=self.stop)
        self.quit_button.pack(pady=20)

        # Lancement du thread de mise à jour
        self.update_thread = threading.Thread(target=self.update_data)
        self.update_thread.daemon = True
        self.update_thread.start()

    def init_modules(self):
        errmsg = YRefParam()
        if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
            print("Impossible d'initialiser les modules Yoctopuce : ", errmsg.value)
            exit(1)

        self.temperature_sensor = YTemperature.FirstTemperature()
        self.relay = YRelay.FindRelay("MXPWRRLY-45C5B.relay1")

        if self.temperature_sensor is None or not self.temperature_sensor.isOnline():
            print("Yocto-Meteo-V2 non détecté. Veuillez le brancher.")
        else:
            print("Yocto-Meteo-V2 détecté.")

        if self.relay is None or not self.relay.isOnline():
            print("Yocto-MaxiPowerRelay (relais MXPWRRLY-45C5B.relay1) non détecté. Veuillez le brancher.")
        else:
            print("Yocto-MaxiPowerRelay (relais MXPWRRLY-45C5B.relay1) détecté.")

    def update_data(self):
        while self.running:
            try:
                # Mise à jour de la liste des appareils pour supporter le hot-plug
                YAPI.UpdateDeviceList(YRefParam())

                # Vérification des capteurs
                if self.temperature_sensor is None or not self.temperature_sensor.isOnline():
                    self.temp_label.config(text="Température : Capteur débranché")
                    self.temperature_sensor = YTemperature.FirstTemperature()
                if self.relay is None or not self.relay.isOnline():
                    self.relay_label.config(text="État du relais : Débranché")
                    self.relay = YRelay.FindRelay("MXPWRRLY-45C5B.relay1")

                # Lecture et mise à jour des données
                if self.temperature_sensor and self.temperature_sensor.isOnline():
                    temperature = self.temperature_sensor.get_currentValue()
                    self.temp_label.config(text=f"Température : {temperature:.1f} °C")

                    if temperature > 25.0:
                        self.relay.set_state(YRelay.STATE_A)  # Relais ouvert
                        self.relay_label.config(text="État du relais : Ouvert")
                    else:
                        self.relay.set_state(YRelay.STATE_B)  # Relais fermé
                        self.relay_label.config(text="État du relais : Fermé")

                time.sleep(1)

            except Exception as e:
                print(f"Erreur : {e}")
                time.sleep(1)

    def stop(self):
        self.running = False
        self.root.destroy()
        YAPI.FreeAPI()


if __name__ == "__main__":
    root = tk.Tk()
    app = YoctoApp(root)
    root.mainloop()
